@@ -16,10 +16,11 @@ use File::Path;
 use File::Copy;
 use File::Find;
 use File::Basename;
+use File::Which qw(which);
 use IPC::Run qw(run);
 use Cwd;
 
-our $VERSION = "0.15";
+our $VERSION = "0.17";
 
 ###########################################
 sub new {
@@ -40,8 +41,8 @@ sub new {
 
     bless $self, $class;
 
-    $self->{tar} = bin_find("tar") unless defined $self->{tar};
-    $self->{tar} = bin_find("gtar") unless defined $self->{tar};
+    $self->{tar} = which("tar") unless defined $self->{tar};
+    $self->{tar} = which("gtar") unless defined $self->{tar};
 
     if( ! defined $self->{tar} ) {
         LOGDIE "tar not found in PATH, please specify location";
@@ -166,9 +167,15 @@ sub add {
     my $binmode = $opts->{binmode} if defined $opts->{binmode};
 
     my $target = File::Spec->catfile($self->{tardir}, $rel_path);
-
     my $target_dir = dirname($target);
-    mkpath($target_dir, 0, 0755) unless -d $target_dir;
+
+    if( ! -d $target_dir ) {
+        if( ref($path_or_stringref) ) {
+            $self->add( dirname( $rel_path ), dirname( $target_dir ) );
+        } else {
+            $self->add( dirname( $rel_path ), dirname( $path_or_stringref ) );
+        }
+    }
 
     if(ref($path_or_stringref)) {
         open FILE, ">$target" or LOGDIE "Can't open $target ($!)";
@@ -177,6 +184,9 @@ sub add {
         }
         print FILE $$path_or_stringref;
         close FILE;
+    } elsif( -d $path_or_stringref ) {
+          # perms will be fixed further down
+        mkpath($target, 0, 0755) unless -d $target;
     } else {
         copy $path_or_stringref, $target or
             LOGDIE "Can't copy $path_or_stringref to $target ($!)";
@@ -402,29 +412,6 @@ sub DESTROY {
     rmtree($self->{tmpdir}) if defined $self->{tmpdir};
 }
 
-######################################
-sub bin_find {
-######################################
-    my($exe) = @_;
-
-    my @paths = split /:/, $ENV{PATH};
-
-    push @paths,
-         "/usr/bin",
-         "/bin",
-         "/usr/sbin",
-         "/opt/bin",
-         "/ops/csw/bin",
-         ;
-
-    for my $path ( @paths ) {
-        my $full = File::Spec->catfile($path, $exe);
-            return $full if -x $full;
-    }
-
-    return undef;
-}
-
 ###########################################
 sub is_gnu {
 ###########################################
@@ -447,8 +434,8 @@ sub ramdisk_mount {
       # mkdir -p /mnt/myramdisk
       # mount -t tmpfs -o size=20m tmpfs /mnt/myramdisk
 
-     $self->{mount}  = bin_find("mount") unless $self->{mount};
-     $self->{umount} = bin_find("umount") unless $self->{umount};
+     $self->{mount}  = which("mount") unless $self->{mount};
+     $self->{umount} = which("umount") unless $self->{umount};
 
      for (qw(mount umount)) {
          if(!defined $self->{$_}) {
